#!/usr/bin/env bash

./zesarux --noconfigfile --machine tbblue --realvideo --enable-watermark --enabletimexvideo --tbblue-fast-boot-mode 

./zesarux --noconfigfile --machine tbblue --realvideo --enable-watermark --enabletimexvideo --tbblue-fast-boot-mode --sna-no-change-machine --enable-esxdos-handler --esxdos-root-dir /Users/cesarhernandez/Downloads/Dungeonette/ /Users/cesarhernandez/Downloads/Dungeonette/project.sna 
./zesarux --noconfigfile --machine tbblue --realvideo --enable-watermark --enabletimexvideo --totalaychips 3 --tbblue-fast-boot-mode --sna-no-change-machine --enable-esxdos-handler --esxdos-root-dir games/3xAY games/3xAY/3xay.sna
./zesarux --noconfigfile --machine tbblue --realvideo --enable-watermark --enabletimexvideo --tbblue-fast-boot-mode --sna-no-change-machine --enable-esxdos-handler --esxdos-root-dir games/Beast games/Beast/lowresdemo.sna
./zesarux --noconfigfile --machine tbblue --realvideo --enable-watermark --enabletimexvideo --tbblue-fast-boot-mode --sna-no-change-machine --enable-esxdos-handler --esxdos-root-dir games/Layer2 games/Layer2/layer2.sna
./zesarux --noconfigfile --machine tbblue --realvideo --enable-watermark --enabletimexvideo --tbblue-fast-boot-mode --sna-no-change-machine --enable-esxdos-handler --esxdos-root-dir games/LowResDemo games/LowResDemo/lowresdemo.sna
