
#ifndef OPTIONS_H
#define OPTIONS_H


#define COMPILE_STDOUT
#define COMPILE_SIMPLETEXT
#define COMPILE_CURSES
#define COMPILE_XWINDOWS
#define COMPILE_SDL
#define COMPILE_FBDEV
#define COMPILE_SSL
#define USE_XEXT
#define USE_XVIDMODE
#define USE_PTHREADS
#define USE_SNDFILE
#define COMPILE_DSP
#define COMPILE_ONEBITSPEAKER
#define COMPILE_ALSA
#define COMPILE_PULSE
#define EMULATE_MEMPTR
#define EMULATE_VISUALMEM
#define EMULATE_CPU_STATS
#define EMULATE_CONTEND
#define PUTPIXELCACHE
#define USE_LINUXREALJOYSTICK

#define COMPILATION_DATE "Thu Sep 25 18:30:39 CEST 2025"
#define COMPILATION_SYSTEM "GNU/Linux"
#define COMPILATION_SYSTEM_RELEASE "Debian GNU/Linux 12 (bookworm)"
#define CONFIGURE_OPTIONS "./configure --enable-ssl --disable-caca --disable-aa --disable-cursesw --prefix /usr"
#define COMPILE_VARIABLES " COMPILE_STDOUT COMPILE_SIMPLETEXT COMPILE_CURSES COMPILE_XWINDOWS COMPILE_SDL COMPILE_FBDEV COMPILE_SSL USE_XEXT USE_XVIDMODE USE_PTHREADS USE_SNDFILE COMPILE_DSP COMPILE_ONEBITSPEAKER COMPILE_ALSA COMPILE_PULSE EMULATE_MEMPTR EMULATE_VISUALMEM EMULATE_CPU_STATS EMULATE_CONTEND PUTPIXELCACHE USE_LINUXREALJOYSTICK"
#define COMPILE_INITIALCFLAGS "-O3"
#define COMPILE_INITIALLDFLAGS "-O3"
#define COMPILE_FINALCFLAGS "-O3 -fsigned-char -I. -Iaudio -Icopy_interfaces -Icores -Icpus -Imachines -Imenu -Isnap -Istorage -Isoundchips -Ithird_party -Ivideo -Ivideo_chips -Izrcp -Izxvision -D_FILE_OFFSET_BITS=64"
#define COMPILE_FINALLDFLAGS "-O3 -lncurses -lssl -lcrypto -lX11 -L/usr/X11R6/lib -L/opt/X11/lib -lXext -lXxf86vm -lpthread -lasound -lpulse -lpulse-simple -lSDL -lsndfile -lpthread"
#define INSTALL_PREFIX "/usr"

#define LINES_SOURCE 390999
#define TOTAL_COMMENTS 65553
#define TOTAL_TODO_ITEMS 2121

#define BUILDNUMBER "1758817835"


#endif

