/*
    ZEsarUX  ZX Second-Emulator And Released for UniX
    Copyright (C) 2013 Cesar Hernandez Bano

    This file is part of ZEsarUX.

    ZEsarUX is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/*

Hi there! This is not really a source code file, this is an easter egg!
Congratulations!

Actually this is a file where I list all the easter eggs in ZEsarUX,
so you can easily try them.

Please, keep the list secret and don't tell anyone,
let's keep the magic :)

So, the list:

1. This file. This is a simple easter egg, but it is one, right? XD

2. Using ZRCP, run the command "easter-egg". See what happens...

3. Using ZRCP, run the command "ATDT". There's a funny answer to that

4. The about menu is different having enabled ZX Desktop than having it disabled. The answer for that change, in the FAQ

5. This one can be fired having ZX Desktop enabled or not:

- ZX Desktop Enabled: Go to the about menu, double click on the image (you need multitasking
for the double click to be detected)

- ZX Desktop Disabled: Go to the about menu, press f key (used to find text).
Just press enter with no text to find. You'll see a cursor in the window,
select the second line and press Enter...

6. Similar to number 5: having ZX Desktop Enabled, go to the about menu, hold and click on the image during 1 second

7. Christmas mode. From December 20 until January 8, there are some changes in the logo and in the welcome message...

*/

